/* int17xx.c: Small interrupt example */

#pragma chip PIC17C766

shrBank char a, b;
bit b1 @ b.1;
bit b2 @ b.2;

bank0 char a0;
bank1 char a1;
bank2 char a2;


/* THIS IS THE RECOMMENDED INTERRUPT ROUTINE STRUCTURE: The macro
   for saving WREG,ALUSTA,BSR and PCLATH is already defined. The
   other registers can be saved on demand: The compiler will
   normally detect the need for this automatically.

#include "int17xxx.h"
#pragma origin 0x8
interrupt iServer( void)
{
    multi_interrupt_entry_and_save

PERIPHERAL_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1
    // the right peripheral interrupt flag must be cleared manually
    /* process periferal interrupt */
    if (RC1IF)  {
        RC1IF = 0;   // application code to be inserted here
    }
    if (TX1IF)  {
        TX1IF = 0;   // application code to be inserted here
    }
    if (CA1IF)  {
        CA1IF = 0;  // application  code to be inserted here
    }
    if (CA2IF)  {
        CA2IF = 0;  // application  code to be inserted here
    }
    if (TMR1IF)  {
        TMR1IF = 0; // application code to be inserted here
    }
    if (TMR2IF)  {
        TMR2IF = 0; // application code to be inserted here
    }
    if (TMR3IF)  {
        TMR3IF = 0; // application code to be inserted here
    }
    if (RBIF)  {
        RBIF = 0;   // application code to be inserted here
    }
    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1
    goto RESTORE_and_return;


TMR0_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    /* process Timer 0 interrupt */
    // T0IF is automatically cleared when the CPU vectors to 0x10
    // application code to be inserted here

    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    goto RESTORE_and_return;


T0CKI_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    /* process T0CKI pin interrupt */
    // T0CKIF is automatically cleared when the CPU vectors to 0x18
    // application code to be inserted here

    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    goto RESTORE_and_return;


INT_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    /* process INT pin interrupt */
    // INTF is automatically cleared when the CPU vectors to 0x8
    // application code to be inserted here

    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1


 RESTORE_and_return:

    interrupt_exit_and_restore
}
*/




/* THIS IS AN EXAMPLE OF HOW TO MAKE CUSTOM REGISTER SAVE FOR
   VERY FAST INTERRUPT HANDLING ON SOME INTERRUPTS.
*/

shrBank char sBSR, sALUSTA;
bank0 char sWREG, sPCLATH;

#pragma origin = 0x8
interrupt INT_server( void)
{
   #asm
    ; ADDRESS 0x08
    ; process INT pin interrupt
    ; INTF is automatically cleared when the CPU vectors to 0x8
    ; 8 application instructions can be inserted here
    MOVFP BSR,sBSR           ; unbanked
    CLRF  BSR
    BSF   b1                 ; set flag
    BSF   PORTB,1            ; pulse on port pin
    NOP
    BCF   PORTB,1
    MOVFP sBSR,BSR
    RETFIE
   #endasm
}

#pragma origin = 0x10
interrupt TMR0_server( void)
{
   #asm
    ; ADDRESS 0x10
    ; TMR0_service: process Timer 0 interrupt
    ; T0IF is automatically cleared when the CPU vectors to 0x10
    ; 8 application instructions can be inserted here
    INFSNZ a      ; 'a' must be unbanked
    RETFIE
    RETFIE
   #endasm
}


#pragma origin = 0x18
interrupt iServer( void)
{
   #asm
    ; ADDRESS 0x18
    MOVFP BSR,sBSR           ; unbanked
    MOVFP ALUSTA, sALUSTA    ; unbanked
    CLRF BSR
    MOVPF PCLATH, sPCLATH    ; bank 0
    MOVWF sWREG              ; bank 0
    CLRF PCLATH
    GOTO T0CKI_service
    NOP

    ; ADDRESS 0x20
    MOVFP BSR,sBSR           ; unbanked
    MOVFP ALUSTA, sALUSTA    ; unbanked
    CLRF BSR
    MOVPF PCLATH, sPCLATH    ; bank 0
    MOVWF sWREG              ; bank 0
    CLRF PCLATH
    ; GOTO PERIPHERAL_service
   #endasm


PERIPHERAL_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1
    // the right peripheral interrupt flag must be cleared manually
    /* process periferal interrupt */
    if (RC1IF)  {
        RC1IF = 0;   // application code to be inserted here
    }
    if (TX1IF)  {
        TX1IF = 0;   // application code to be inserted here
    }
    if (CA1IF)  {
        CA1IF = 0;  // application  code to be inserted here
    }
    if (CA2IF)  {
        CA2IF = 0;  // application  code to be inserted here
    }
    if (TMR1IF)  {
        TMR1IF = 0; // application code to be inserted here
    }
    if (TMR2IF)  {
        TMR2IF = 0; // application code to be inserted here
    }
    if (TMR3IF)  {
        TMR3IF = 0; // application code to be inserted here
    }
    if (RBIF)  {
        RBIF = 0;   // application code to be inserted here
    }
    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

   #asm
    MOVFP sWREG,W            ; bank 0
    MOVFP sPCLATH, PCLATH    ; bank 0
    MOVFP sALUSTA, ALUSTA    ; unbanked
    MOVFP sBSR, BSR          ; unbanked
    RETFIE
   #endasm


T0CKI_service:
    // save on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

    /* process T0CKI pin interrupt */
    // T0CKIF is automatically cleared when the CPU vectors to 0x18
    // application code to be inserted here

    // restore on demand: PRODL,PRODH,TBLPTRH,TBLPTRL,FSR0,FSR1

   #asm
    MOVFP sWREG,W            ; bank 0
    MOVFP sPCLATH, PCLATH    ; bank 0
    MOVFP sALUSTA, ALUSTA    ; unbanked
    MOVFP sBSR, BSR          ; unbanked
    RETFIE
   #endasm
}






void main( void)
{
    a = 0;
    while (1)
        ;
}

